package com.tzld.piaoquan.risk.control.config;

import com.alibaba.fastjson.JSON;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.tzld.piaoquan.risk.control.common.annotation.UnAuth;
import com.tzld.piaoquan.risk.control.common.enums.ExceptionEnum;
import com.tzld.piaoquan.risk.control.common.enums.RedisPrefixEnum;
import com.tzld.piaoquan.risk.control.common.exception.CommonException;
import com.tzld.piaoquan.risk.control.component.RedisUtils;
import com.tzld.piaoquan.risk.control.model.po.User;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Objects;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Component
public class AuthInterceptor implements HandlerInterceptor {

    private final static Logger log = LoggerFactory.getLogger(AuthInterceptor.class);

    ThreadPoolExecutor executor = new ThreadPoolExecutor(128, 128,
            0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(100000), new ThreadFactoryBuilder().setNameFormat("ad-platform-login-service-pool-%d").build(), new ThreadPoolExecutor.AbortPolicy());

    @Autowired
    private RedisUtils redisUtils;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if ((StringUtils.isNotBlank(request.getRequestURI()) && request.getRequestURI().contains("auth")) ||
                (Objects.nonNull(request.getRequestURL()) && request.getRequestURL().toString().contains("auth"))
        ) {
            return Boolean.TRUE;
        }

        if (HttpMethod.OPTIONS.name().equals(request.getMethod())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }

        // 忽略带UnAuth注解的请求, 不做后续token认证校验
        if (handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            UnAuth jwtIgnore = handlerMethod.getMethodAnnotation(UnAuth.class);
            if (jwtIgnore != null) {
                return Boolean.TRUE;
            }
        }
        final String token = request.getHeader("token");

        if (StringUtils.isBlank(token)) {
            log.info("LOGIN_ERROR, authHeader:{}", token);
            throw new CommonException(ExceptionEnum.LOGIN_ERROR, "未登录，请先登录！！！");
        }

        this.validToken(token);

        return Boolean.TRUE;
    }

    private void validToken(String token) {
        String loginUserString = redisUtils.getString(RedisPrefixEnum.RISK_CONTROL_TOKEN.getPrefix().replace("{token}", token));
        if (StringUtils.isBlank(loginUserString)) {
            log.info("LOGIN_ERROR-1, authHeader:{}", token);
            throw new CommonException(ExceptionEnum.LOGIN_ERROR, "登录失效，请重新登录。");
        }
        User user = null;
        try {
            user = JSON.parseObject(loginUserString, User.class);
        } catch (Exception e) {
            log.info("LOGIN_ERROR-2, authHeader:{}", token);
            throw new CommonException(ExceptionEnum.LOGIN_ERROR, "登录失效，请重新登录。");
        }
        // token验证完成后获取对应信息，存放在本地线程中
        LoginUserContext.setLoginUser(user);
        // 异步处理续期
        renewalToken(token);
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        LoginUserContext.remove();
    }

    private void renewalToken(String token) {
        executor.execute(() -> {
            String key = RedisPrefixEnum.RISK_CONTROL_TOKEN.getPrefix().replace("{token}", token);
            Long seconds = redisUtils.getKeyExpire(key);
            if (seconds < 600) {
                redisUtils.expire(key, 7200L);
            }
        });
    }
}